@@ -109,6 +109,12 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      * format if needed. 
      */
     ICON("icon"),
+    /**
+     * The key for the SHA-1 digest of the icon file of a piece of furniture (optional). 
+     * This property is used to compare faster catalog resources with the ones of a read home,
+     * and should be encoded in Base64.  
+     */
+    ICON_DIGEST("iconDigest"),
     /**
      * The key for the plan icon file of a piece of furniture (optional).
      * This icon file can be either the path to an image relative to classpath
@@ -116,6 +122,12 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      * format if needed.
      */
     PLAN_ICON("planIcon"),
+    /**
+     * The key for the SHA-1 digest of the plan icon file of a piece of furniture (optional). 
+     * This property is used to compare faster catalog resources with the ones of a read home,
+     * and should be encoded in Base64.  
+     */
+    PLAN_ICON_DIGEST("planIconDigest"),
     /**
      * The key for the 3D model file of a piece of furniture (mandatory).
      * The 3D model file can be either a path relative to classpath
@@ -123,6 +135,12 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      * format if needed.
      */
     MODEL("model"),
+    /**
+     * The key for the SHA-1 digest of the 3D model file of a piece of furniture (optional). 
+     * This property is used to compare faster catalog resources with the ones of a read home,
+     * and should be encoded in Base64.  
+     */
+    MODEL_DIGEST("modelDigest"),
     /**
      * The key for a piece of furniture with multiple parts (optional).
      * If the value of this key is <code>true</code>, all the files
@@ -233,6 +251,12 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      * The key for the elevation in centimeters of a piece of furniture (optional).
      */
     ELEVATION("elevation"),
+    /**
+     * The key for the preferred elevation (from the bottom of a piece) at which should be placed  
+     * an object dropped on a piece (optional). A negative value means that the piece should be ignored
+     * when an object is dropped on it. By default, this elevation is equal to its height. 
+     */
+    DROP_ON_TOP_ELEVATION("dropOnTopElevation"),
     /**
      * The key for the transformation matrix values applied to a piece of furniture (optional).
      * If the 3D model of a piece of furniture isn't correctly oriented, 
@@ -578,17 +602,18 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     if (gradeString != null) {
       grade = Float.valueOf(gradeString);
     }
-    Content icon  = getContent(resource, PropertyKey.ICON.getKey(index), 
+    Content icon  = getContent(resource, PropertyKey.ICON.getKey(index), PropertyKey.ICON_DIGEST.getKey(index), 
         furnitureCatalogUrl, furnitureResourcesUrlBase, false, false);
-    Content planIcon = getContent(resource, PropertyKey.PLAN_ICON.getKey(index), 
+    Content planIcon = getContent(resource, PropertyKey.PLAN_ICON.getKey(index), PropertyKey.PLAN_ICON_DIGEST.getKey(index), 
         furnitureCatalogUrl, furnitureResourcesUrlBase, false, true);
     boolean multiPartModel = getOptionalBoolean(resource, PropertyKey.MULTI_PART_MODEL.getKey(index), false);
-    Content model = getContent(resource, PropertyKey.MODEL.getKey(index), 
+    Content model = getContent(resource, PropertyKey.MODEL.getKey(index), PropertyKey.MODEL_DIGEST.getKey(index), 
         furnitureCatalogUrl, furnitureResourcesUrlBase, multiPartModel, false);
     float width = Float.parseFloat(resource.getString(PropertyKey.WIDTH.getKey(index)));
     float depth = Float.parseFloat(resource.getString(PropertyKey.DEPTH.getKey(index)));
     float height = Float.parseFloat(resource.getString(PropertyKey.HEIGHT.getKey(index)));
     float elevation = getOptionalFloat(resource, PropertyKey.ELEVATION.getKey(index), 0);
+    float dropOnTopElevation = getOptionalFloat(resource, PropertyKey.DROP_ON_TOP_ELEVATION.getKey(index), height) / height;
     boolean movable = Boolean.parseBoolean(resource.getString(PropertyKey.MOVABLE.getKey(index)));
     boolean doorOrWindow = Boolean.parseBoolean(resource.getString(PropertyKey.DOOR_OR_WINDOW.getKey(index)));
     String staircaseCutOutShape = getOptionalString(resource, PropertyKey.STAIRCASE_CUT_OUT_SHAPE.getKey(index), null);     
@@ -620,19 +645,19 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
           resource, PropertyKey.DOOR_OR_WINDOW_WALL_DISTANCE.getKey(index), 0) / depth;
       Sash [] sashes = getDoorOrWindowSashes(resource, index, width, depth);
       return new CatalogDoorOrWindow(id, name, description, information, tags, creationDate, grade, 
-          icon, planIcon, model, width, depth, height, elevation, movable, 
+          icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable, 
           doorOrWindowCutOutShape, wallThicknessPercentage, wallDistancePercentage, sashes,
           modelRotation, creator, resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);
     } else {
       LightSource [] lightSources = getLightSources(resource, index, width, depth, height);
       if (lightSources != null) {
         return new CatalogLight(id, name, description, information, tags, creationDate, grade, 
-            icon, planIcon, model, width, depth, height, elevation, movable, 
+            icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable, 
             lightSources, staircaseCutOutShape, modelRotation, creator, 
             resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);
       } else {
         return new CatalogPieceOfFurniture(id, name, description, information, tags, creationDate, grade, 
-            icon, planIcon, model, width, depth, height, elevation, movable, 
+            icon, planIcon, model, width, depth, height, elevation, dropOnTopElevation, movable, 
             staircaseCutOutShape, modelRotation, creator, 
             resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);
       }
@@ -652,7 +677,8 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
   /**
    * Returns a valid content instance from the resource file or URL value of key.
    * @param resource a resource bundle
-   * @param contentKey  the key of a resource content file
+   * @param contentKey        the key of a resource content file
+   * @param contentDigestKey  the key of the digest of a resource content file
    * @param furnitureUrl the URL of the file containing the target resource if it's not <code>null</code> 
    * @param resourceUrlBase the URL used as a base to build the URL to content file  
    *            or <code>null</code> if it's read from current classpath or <code>furnitureCatalogUrl</code>.
@@ -662,16 +688,18 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
    */
   private Content getContent(ResourceBundle resource, 
                              String contentKey, 
+                             String contentDigestKey,
                              URL furnitureUrl,
                              URL resourceUrlBase, 
                              boolean multiPartModel,
                              boolean optional) {
     String contentFile = optional
-       ? getOptionalString(resource, contentKey, null)
-       : resource.getString(contentKey);
+        ? getOptionalString(resource, contentKey, null)
+        : resource.getString(contentKey);
     if (optional && contentFile == null) {
       return null;
     }
+    URLContent content;
     try {
       // Try first to interpret contentFile as an absolute URL 
       // or an URL relative to resourceUrlBase if it's not null
@@ -686,19 +714,33 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
           url = new URL("jar:" + url);
         }
       }
-      return new URLContent(url);
+      content = new URLContent(url);
     } catch (MalformedURLException ex) {
       if (furnitureUrl == null) {
         // Otherwise find if it's a resource
-        return new ResourceURLContent(DefaultFurnitureCatalog.class, contentFile, multiPartModel);
+        content = new ResourceURLContent(DefaultFurnitureCatalog.class, contentFile, multiPartModel);
       } else {
         try {
-          return new ResourceURLContent(new URL("jar:" + furnitureUrl + "!" + contentFile), multiPartModel);
+          content = new ResourceURLContent(new URL("jar:" + furnitureUrl + "!" + contentFile), multiPartModel);
         } catch (MalformedURLException ex2) {
           throw new IllegalArgumentException("Invalid URL", ex2);
         }
       }
     }
+    
+    // Store content digest if it exists
+    // Except in special cases like URL content in applets where it might avoid to download content  
+    // to compute its digest, it's not recommended to store digests in sh3f and imported files. 
+    // Missing digests will be computed on demand, ensuring it will be updated in case content is damaged
+    String contentDigest = getOptionalString(resource, contentDigestKey, null);
+    if (contentDigest != null && contentDigest.length() > 0) {
+      try {        
+        ContentDigestManager.getInstance().setContentDigest(content, Base64.decode(contentDigest));
+      } catch (IOException ex) {
+        // Ignore wrong digest
+      }
+    }
+    return content;
   }
   
   /**
