@@ -30,16 +30,18 @@ import java.io.OutputStreamWriter;
 import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLEncoder;
-import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedHashMap;
-import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipOutputStream;
 
 import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.tools.ResourceURLContent;
+import com.eteks.sweethome3d.tools.SimpleURLContent;
 import com.eteks.sweethome3d.tools.TemporaryURLContent;
 import com.eteks.sweethome3d.tools.URLContent;
 
@@ -47,21 +49,26 @@ import com.eteks.sweethome3d.tools.URLContent;
  * An <code>OutputStream</code> filter that writes a home in a stream 
  * at .sh3d file format. 
  * @see DefaultHomeInputStream
+ * @author Emmanuel Puybaret
  */
 public class DefaultHomeOutputStream extends FilterOutputStream {
   private int              compressionLevel;
   private ContentRecording contentRecording;
+  private boolean          serializedHome;
+  private HomeXMLExporter  homeXmlExporter;
   
   /**
-   * Creates a stream that will serialize a home and all the contents it references
-   * in an uncompressed zip stream.
+   * Creates a stream that will save a home and all the contents it references
+   * in an uncompressed zip stream. Home data will be serialized in an entry named
+   * <code>Home</code>.
    */
   public DefaultHomeOutputStream(OutputStream out) throws IOException {
     this(out, 0, false);
   }
 
   /**
-   * Creates a stream that will serialize a home in a zip stream.
+   * Creates a stream that will save a home in a zip stream. Home data will be serialized 
+   * in an entry named <code>Home</code>.
    * @param compressionLevel 0-9
    * @param includeTemporaryContent if <code>true</code>, content instances of 
    *            <code>TemporaryURLContent</code> class referenced by the saved home 
@@ -79,16 +86,41 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
   }
 
   /**
-   * Creates a stream that will serialize a home in a zip stream.
+   * Creates a stream that will save a home in a zip stream. Home data will be serialized 
+   * in an entry named <code>Home</code>.
    * @param compressionLevel 0-9
    * @param contentRecording how content should be recorded with home.  
    */
   public DefaultHomeOutputStream(OutputStream out,
                                  int          compressionLevel, 
                                  ContentRecording contentRecording) throws IOException {
+    this(out, compressionLevel, contentRecording, true, null);
+  }
+
+  /**
+   * Creates a stream that will serialize a home in a zip stream. Home data will be serialized 
+   * in an entry named <code>Home</code> if <code>serializedHome</code> is <code>true</code>, 
+   * and saved in <code>Home.xml</code> entry at XML format if <code>homeXmlExporter</code> is not <code>null</code>.
+   * @param compressionLevel 0-9
+   * @param contentRecording specifies how content should be recorded with home  
+   * @param serializedHome if <code>true</code>, zip stream will include a <code>Home</code> 
+   *            entry containing the serialized home
+   * @param homeXmlExporter  if not <code>null</code>, sets how a home will be saved
+   *            in an additional <code>Home.xml</code> entry
+   */
+  public DefaultHomeOutputStream(OutputStream out,
+                                 int          compressionLevel, 
+                                 ContentRecording contentRecording,
+                                 boolean          serializedHome,
+                                 HomeXMLExporter  homeXmlExporter) throws IOException {
     super(out);
+    if (!serializedHome && homeXmlExporter == null) {
+      throw new IllegalArgumentException("No entry specified for home data");
+    }
     this.compressionLevel = compressionLevel;
     this.contentRecording = contentRecording;
+    this.serializedHome = serializedHome;
+    this.homeXmlExporter = homeXmlExporter;
   }
 
   /**
@@ -111,46 +143,84 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
     ZipOutputStream zipOut = new ZipOutputStream(this.out);
     zipOut.setLevel(this.compressionLevel);
     checkCurrentThreadIsntInterrupted();
-    // Write home in the first entry named "Home"
-    zipOut.putNextEntry(new ZipEntry("Home"));
-    // Use an ObjectOutputStream that keeps track of Content objects
-    HomeObjectOutputStream objectOut = new HomeObjectOutputStream(zipOut);
-    objectOut.writeObject(home);
-    objectOut.flush();
-    zipOut.closeEntry();
-    List<Content> savedContents = objectOut.getSavedContents();
-    if (savedContents.size() > 0) {
-      // In the second entry named "ContentDigests", write content digests to help repair damaged files     
+    // Track content that must be saved in the zip stream with a dummy output stream 
+    HomeContentObjectsTracker contentTracker = new HomeContentObjectsTracker(new OutputStream() {
+        @Override
+        public void write(int b) throws IOException {
+          // Don't write anything
+        }
+      });
+    contentTracker.writeObject(home);
+    Map<Content, String> savedContentNames = contentTracker.getSavedContentNames();
+
+    if (this.serializedHome) {
+      // Write home in the first entry named "Home"
+      zipOut.putNextEntry(new ZipEntry("Home"));
+      // Save home replacing Content objects if needed
+      HomeObjectOutputStream objectOut = new HomeObjectOutputStream(zipOut, savedContentNames);
+      objectOut.writeObject(home);
+      objectOut.flush();
+      zipOut.closeEntry();
+    }
+
+    if (this.homeXmlExporter != null) {
+      // Write home at XML format in the second entry named "Home.xml"
+      zipOut.putNextEntry(new ZipEntry("Home.xml"));
+      // Save home replacing Content objects if needed
+      XMLWriter xmlWriter = new XMLWriter(zipOut);
+      this.homeXmlExporter.setSavedContentNames(savedContentNames);
+      this.homeXmlExporter.writeElement(xmlWriter, home);
+      xmlWriter.flush();
+      zipOut.closeEntry();
+    }
+    
+    if (savedContentNames.size() > 0) {
+      Set<String> contentEntryNames = new HashSet<String>(); 
+      // In the next entry named "ContentDigests", write content digests to help repair damaged files     
       zipOut.putNextEntry(new ZipEntry("ContentDigests"));
       OutputStreamWriter writer = new OutputStreamWriter(zipOut, "UTF-8");
       ContentDigestManager digestManager = ContentDigestManager.getInstance();
       writer.write("ContentDigests-Version: 1.0\n\n");
-      for (Content content : savedContents) {
-        writer.write("Name: " + objectOut.getContentEntry(content) + "\n");
-        writer.write("SHA-1-Digest: " + Base64.encodeBytes(digestManager.getContentDigest(content)) + "\n\n");
+      for (Map.Entry<Content, String> savedContent : savedContentNames.entrySet()) {
+        String contentEntryName = savedContent.getValue();
+        if (!contentEntryNames.contains(contentEntryName)) {
+          contentEntryNames.add(contentEntryName);
+          writer.write("Name: " + contentEntryName + "\n");
+          writer.write("SHA-1-Digest: " + Base64.encodeBytes(digestManager.getContentDigest(savedContent.getKey())) + "\n\n");
+        }
       }
       writer.flush();
       zipOut.closeEntry();
-    }
-    // Write Content objects in files "0" to "n - 1"
-    int i = 0;
-    for (Content content : savedContents) {
-      String entryNameOrDirectory = String.valueOf(i++);
-      if (content instanceof ResourceURLContent) {
-        writeResourceZipEntries(zipOut, entryNameOrDirectory, (ResourceURLContent)content);
-      } else if (content instanceof URLContent
-                 && ((URLContent)content).isJAREntry()) {
-        URLContent urlContent = (URLContent)content;
-        // If content comes from a home stream
-        if (urlContent instanceof HomeURLContent) {
-          writeHomeZipEntries(zipOut, entryNameOrDirectory, (HomeURLContent)urlContent);            
-        } else {
-          writeZipEntries(zipOut, entryNameOrDirectory, urlContent);
+    
+      // Write Content objects in additional zip entries
+      contentEntryNames.clear(); 
+      for (Map.Entry<Content, String> savedContent : savedContentNames.entrySet()) {
+        String contentEntryName = savedContent.getValue();
+        if (!contentEntryNames.contains(contentEntryName)) {
+          contentEntryNames.add(contentEntryName);
+          Content content = savedContent.getKey();
+          int slashIndex = contentEntryName.indexOf('/');
+          if (slashIndex > 0) {
+            contentEntryName = contentEntryName.substring(0, slashIndex);
+          }
+          if (content instanceof ResourceURLContent) {
+            writeResourceZipEntries(zipOut, contentEntryName, (ResourceURLContent)content);
+          } else if (content instanceof URLContent
+                     && !(content instanceof SimpleURLContent)
+                     && ((URLContent)content).isJAREntry()) {
+            URLContent urlContent = (URLContent)content;
+            // If content comes from a home stream
+            if (urlContent instanceof HomeURLContent) {
+              writeHomeZipEntries(zipOut, contentEntryName, (HomeURLContent)urlContent);            
+            } else {
+              writeZipEntries(zipOut, contentEntryName, urlContent);
+            }
+          } else {
+            writeZipEntry(zipOut, contentEntryName, content);
+          }
         }
-      } else {
-        writeZipEntry(zipOut, entryNameOrDirectory, content);
-      }
-    }  
+      }  
+    }
     // Finish zip writing
     zipOut.finish();
   }
@@ -271,13 +341,14 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
   }
 
   /**
-   * <code>ObjectOutputStream</code> that replaces <code>Content</code> objects
-   * by temporary <code>URLContent</code> objects and stores them in a list.
+   * A dummy <code>ObjectOutputStream</code> that keeps track of the <code>Content</code> 
+   * objects of a home that should be saved.
    */
-  private class HomeObjectOutputStream extends ObjectOutputStream {
-    private Map<Content, URLContent> savedContents = new LinkedHashMap<Content, URLContent>();
+  private class HomeContentObjectsTracker extends ObjectOutputStream {
+    private Map<Content, String> savedContentNames = new LinkedHashMap<Content, String>();
+    private int savedContentIndex = 0;
 
-    public HomeObjectOutputStream(OutputStream out) throws IOException {
+    public HomeContentObjectsTracker(OutputStream out) throws IOException {
       super(out);
       if (contentRecording != ContentRecording.INCLUDE_NO_CONTENT) {
         enableReplaceObject(true);
@@ -294,9 +365,10 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
           URLContent urlContent = (URLContent)obj;
           // Check if duplicated content can be avoided 
           ContentDigestManager contentDigestManager = ContentDigestManager.getInstance();
-          for (Map.Entry<Content, URLContent> contentEntry : savedContents.entrySet()) {
+          for (Map.Entry<Content, String> contentEntry : this.savedContentNames.entrySet()) {
             if (contentDigestManager.equals(urlContent, contentEntry.getKey())) {
-              return contentEntry.getValue();
+              this.savedContentNames.put((Content)obj, contentEntry.getValue());
+              return obj;
             }
           }
           checkCurrentThreadIsntInterrupted();
@@ -320,7 +392,7 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
                   subEntryName = entryName.substring(lastSlashIndex);
                 }
               }
-            } else {
+            } else if (!(urlContent instanceof SimpleURLContent)) {
               // Retrieve entry name in zipped stream
               subEntryName = "/" + entryName;
             }            
@@ -340,28 +412,54 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
           }
         } 
 
-        // Return a temporary URL that points to content object 
-        URLContent tmpContent = new URLContent(new URL("jar:file:temp!/" + this.savedContents.size() + subEntryName));
-        // Add obj to Content objects list
-        this.savedContents.put((Content)obj, tmpContent);
-        return tmpContent;
-      } else {
-        return obj;
-      }
+        // Build a relative URL that points to content object 
+        String homeContentPath = this.savedContentIndex++ + subEntryName;
+        this.savedContentNames.put((Content)obj, homeContentPath);
+      } 
+      return obj;
     }
     
     /**
-     * Returns the contents to be saved.
+     * Returns the names of the home contents to be saved.
      */
-    public List<Content> getSavedContents() {
-      return new ArrayList<Content>(this.savedContents.keySet());
+    public Map<Content, String> getSavedContentNames() {
+      return this.savedContentNames;
+    }
+  }
+  
+  /**
+   * <code>ObjectOutputStream</code> that replaces <code>Content</code> objects
+   * by temporary <code>URLContent</code> objects and stores them in a list.
+   */
+  private class HomeObjectOutputStream extends ObjectOutputStream {
+    private Map<Content, String>    savedContentNames;
+    private Map<String, URLContent> replacedContents = new HashMap<String, URLContent>();
+
+    public HomeObjectOutputStream(OutputStream out,
+                                  Map<Content, String> savedContentNames) throws IOException {
+      super(out);
+      this.savedContentNames = savedContentNames;
+      if (contentRecording != ContentRecording.INCLUDE_NO_CONTENT) {
+        enableReplaceObject(true);
+      }
     }
 
-    /**
-     * Returns the entry of a given <code>content</code>.
-     */
-    public String getContentEntry(Content content) {
-      return this.savedContents.get(content).getJAREntryName();
+    @Override
+    protected Object replaceObject(Object obj) throws IOException {
+      if (obj instanceof Content) {
+        String savedContentName = this.savedContentNames.get((Content)obj);
+        if (savedContentName != null) {
+          checkCurrentThreadIsntInterrupted();
+          // Ensure that the duplicated content share the same replaced URLContent instance
+          URLContent replacedContent = this.replacedContents.get(savedContentName);
+          if (replacedContent == null) {
+            replacedContent = new URLContent(new URL("jar:file:temp!/" + savedContentName));
+            this.replacedContents.put(savedContentName, replacedContent);
+          }
+          return replacedContent;          
+        }
+      }
+      return obj;
     }
   }
 }
\ No newline at end of file
