@@ -118,7 +118,7 @@ public class Ground3D extends Object3DBranch {
       groundAppearance.getTransparencyAttributes().setTransparencyMode(TransparencyAttributes.NONE);      
     } else {
       groundAppearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_COLOR, 0));
-      groundAppearance.setTextureAttributes(getTextureAttributes(groundTexture));
+      groundAppearance.setTextureAttributes(getTextureAttributes(groundTexture, true));
       final TextureManager textureManager = TextureManager.getInstance();
       textureManager.loadTexture(groundTexture.getImage(), waitTextureLoadingEnd,
           new TextureManager.TextureObserver() {
@@ -348,23 +348,13 @@ public class Ground3D extends Object3DBranch {
           ? new TexCoord2f [vertexCount]
           : null;
       
-      float textureWidth;
-      float textureHeight;
-      if (groundTexture != null) {
-        textureWidth = groundTexture.getWidth();
-        textureHeight = groundTexture.getHeight();
-      } else {
-        textureWidth = 0;
-        textureHeight = 0;
-      }
       int j = 0;
       for (float [][] areaPartPoints : areaPoints) {
         for (int i = 0; i < areaPartPoints.length; i++, j++) {
           float [] point = areaPartPoints [i];
           geometryCoords [j] = new Point3f(point [0], elevation, point [1]);
           if (groundTexture != null) {
-            geometryTextureCoords [j] = new TexCoord2f((point [0] - this.originX) / textureWidth, 
-                (this.originY - point [1]) / textureHeight);
+            geometryTextureCoords [j] = new TexCoord2f(point [0] - this.originX, this.originY - point [1]);
           }
         }
       }
@@ -393,15 +383,6 @@ public class Ground3D extends Object3DBranch {
     TexCoord2f [] geometryTextureCoords = groundTexture != null 
         ? new TexCoord2f [geometryCoords.length]
         : null;
-    float textureWidth;
-    float textureHeight;
-    if (groundTexture != null) {
-      textureWidth = groundTexture.getWidth();
-      textureHeight = groundTexture.getHeight();
-    } else {
-      textureWidth = 0;
-      textureHeight = 0;
-    }
     for (int i = 0, j = 0; i < areaPoints.length; i++) {
       float [] point = areaPoints [i];
       float [] nextPoint = areaPoints [i < areaPoints.length - 1 ? i + 1 : 0];
@@ -411,10 +392,10 @@ public class Ground3D extends Object3DBranch {
       geometryCoords [j++] = new Point3f(nextPoint [0], elevation, nextPoint [1]);
       if (groundTexture != null) {
         float distance = (float)Point2D.distance(point [0], point [1], nextPoint [0], nextPoint [1]);
-        geometryTextureCoords [j - 4] = new TexCoord2f(point [0] / textureWidth, elevation / textureHeight);
-        geometryTextureCoords [j - 3] = new TexCoord2f(point [0] / textureWidth, (elevation + sideHeight) / textureHeight);
-        geometryTextureCoords [j - 2] = new TexCoord2f((point [0] - distance) / textureWidth, (elevation + sideHeight) / textureHeight);
-        geometryTextureCoords [j - 1] = new TexCoord2f((point [0] - distance) / textureWidth, elevation / textureHeight);
+        geometryTextureCoords [j - 4] = new TexCoord2f(point [0], elevation);
+        geometryTextureCoords [j - 3] = new TexCoord2f(point [0], elevation + sideHeight);
+        geometryTextureCoords [j - 2] = new TexCoord2f(point [0] - distance, elevation + sideHeight);
+        geometryTextureCoords [j - 1] = new TexCoord2f(point [0] - distance, elevation);
       }
     }
 
