@@ -854,7 +854,7 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
     byte [][] filterCollationKeys = recentFilters.get(filter);
     if (filterCollationKeys == null) {
       // Each substring in filter is a search criterion that must be verified 
-      String [] filterCriteria = filter.split("\\s|\\p{Punct}");
+      String [] filterCriteria = filter.split("\\s|\\p{Punct}|\\|");
       List<byte []> filterCriteriaCollationKeys = new ArrayList<byte []>(filterCriteria.length);
       for (String criterion : filterCriteria) {
         if (criterion.length() > 0) {
@@ -880,22 +880,22 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
       // (collect the name, category, creator, description and tags of each piece)
       StringBuilder search = new StringBuilder();
       search.append(getName());
-      search.append('\t');
+      search.append('|');
       if (getCategory() != null) {
         search.append(getCategory().getName());
-        search.append('\t');
+        search.append('|');
       }
       if (getCreator() != null) {
         search.append(getCreator());
-        search.append('\t');
+        search.append('|');
       }
       if (getDescription() != null) {
         search.append(getDescription());
-        search.append('\t');
+        search.append('|');
       }
       for (String tag : getTags()) {
         search.append(tag);
-        search.append('\t');
+        search.append('|');
       }
       
       this.filterCollationKey = COMPARATOR.getCollationKey(search.toString()).toByteArray();
