@@ -25,8 +25,8 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URISyntaxException;
-
-import junit.framework.TestCase;
+import java.net.URL;
+import java.util.Arrays;
 
 import com.eteks.sweethome3d.io.DefaultFurnitureCatalog;
 import com.eteks.sweethome3d.io.DefaultUserPreferences;
@@ -35,10 +35,16 @@ import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.DamagedHomeRecorderException;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.Home;
+import com.eteks.sweethome3d.model.HomeFurnitureGroup;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeRecorder;
+import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.RecorderException;
+import com.eteks.sweethome3d.model.TextStyle;
 import com.eteks.sweethome3d.model.Wall;
+import com.eteks.sweethome3d.tools.URLContent;
+
+import junit.framework.TestCase;
 
 /**
  * Tests {@link HomeFileRecorder} class.
@@ -46,35 +52,43 @@ import com.eteks.sweethome3d.model.Wall;
  */
 public class HomeFileRecorderTest extends TestCase {
   public void testWriteReadHome() throws RecorderException {
-    // 1. Create an empty home
+    // Create an empty home with a wall and a piece of furniture
     Home home1 = new Home();
-    // Add to home a wall and a piece of furniture
     Wall wall = new Wall(0, 10, 100, 80, 10, home1.getWallHeight());
+    wall.setProperty("id", "wall1");
     home1.addWall(wall);
     FurnitureCatalog catalog = new DefaultFurnitureCatalog();
     HomePieceOfFurniture piece = new HomePieceOfFurniture(
         catalog.getCategories().get(0).getFurniture().get(0));
+    piece.setProperty("id", "piece1");
+    piece.setProperty("name", "value");
     home1.addPieceOfFurniture(piece);
     
-    // 2. Record home in a file named test.sh3d in current directory
-    HomeRecorder recorder = new HomeFileRecorder();
+    // Test if home is correctly saved
+    checkSavedHome(home1, new HomeFileRecorder());
+    // Test if home with XML entry is correctly saved
+    checkSavedHome(home1, new HomeFileRecorder(9, false, null, false, true));
+  }
+  
+  private void checkSavedHome(Home home, HomeRecorder recorder) throws RecorderException {
+    // 1. Record home in a file named test.sh3d in current directory
     String testFile = new File("test.sh3d").getAbsolutePath();
-    recorder.writeHome(home1, testFile); 
+    recorder.writeHome(home, testFile); 
     // Check test.sh3d file exists
     assertTrue("File test.sh3d doesn't exist", recorder.exists(testFile));
     
-    // 3. Read test.sh3d file in a new home
-    Home home2 = recorder.readHome(testFile);
+    // 2. Read test.sh3d file in a new home
+    Home readHome = recorder.readHome(testFile);
     // Compare home content
-    assertNotSame("Home not loaded", home1, home2);
+    assertNotSame("Home not loaded", home, readHome);
     assertEquals("Home wall height", 
-        home1.getWallHeight(), home2.getWallHeight());
+        home.getWallHeight(), readHome.getWallHeight());
     assertEquals("Home walls wrong count", 
-        home1.getWalls().size(), home2.getWalls().size());
-    assertEquals(wall, home2.getWalls().iterator().next());
+        home.getWalls().size(), readHome.getWalls().size());
+    assertEquals(home.getWalls().iterator().next(), readHome.getWalls().iterator().next());
     assertEquals("Home furniture wrong count", 
-        home1.getFurniture().size(), home2.getFurniture().size());
-    assertEquals(piece, home2.getFurniture().get(0));
+        home.getFurniture().size(), readHome.getFurniture().size());
+    assertEquals(home.getFurniture().iterator().next(), readHome.getFurniture().get(0));
 
     // Delete file
     if (!new File(testFile).delete()) {
@@ -82,6 +96,55 @@ public class HomeFileRecorderTest extends TestCase {
     }
   }
   
+  public void testXMLEntryConsistency() throws URISyntaxException, RecorderException, IOException {
+    checkXMLEntryConsistency(new File(HomeControllerTest.class.getResource("resources/home1.sh3d").toURI()));
+    
+    // Create a home with properties and text style set on group items
+    Home home = new Home();
+    Level level0 = new Level("Level0", 0, 0.1f, 250f);
+    home.addLevel(level0);
+    Level level1 = new Level("Level1", 252f, 2f, 250f);
+    level1.setViewable(false);
+    home.addLevel(level1);
+    home.setSelectedLevel(level0);
+    home.addWall(new Wall(0, 10, 100, 80, 10, Float.NaN)); // Use NaN to check how it will be written then read
+    FurnitureCatalog catalog = new DefaultFurnitureCatalog();
+    HomePieceOfFurniture piece1 = new HomePieceOfFurniture(catalog.getCategories().get(0).getFurniture().get(0));
+    piece1.setProperty("id", "piece1");
+    HomePieceOfFurniture piece2 = new HomePieceOfFurniture(catalog.getCategories().get(0).getFurniture().get(1));
+    piece2.setProperty("id", "piece2");
+    piece2.setNameStyle(new TextStyle(12, false, false));
+    HomeFurnitureGroup group = new HomeFurnitureGroup(Arrays.asList(piece1, piece2), "Group");
+    group.setProperty("test", "value");
+    home.addPieceOfFurniture(group);
+    // Save home with Home serialized entry
+    File savedFileWithHomeEntry = File.createTempFile("test", ".sh3d");
+    new HomeFileRecorder(0, false, null, false, false).writeHome(home, savedFileWithHomeEntry.getAbsolutePath());
+    checkXMLEntryConsistency(savedFileWithHomeEntry);
+    savedFileWithHomeEntry.delete();
+  }
+  
+  public void checkXMLEntryConsistency(File homeFile) throws RecorderException, IOException {
+    HomeRecorder homeEntryRecorder = new HomeFileRecorder(0, false, null, false, false);
+    Home home = homeEntryRecorder.readHome(homeFile.getAbsolutePath());
+    assertTrue("Tested home has no walls", home.getWalls().size() > 0);
+    assertTrue("Tested home has no furniture", home.getFurniture().size() > 0);
+    // Save home with an XML entry
+    HomeRecorder homeXmlEntryRecorder = new HomeFileRecorder(0, false, null, false, true);
+    File savedFileWithXmlEntry = File.createTempFile("homeXML", ".sh3d");
+    homeXmlEntryRecorder.writeHome(home, savedFileWithXmlEntry.getAbsolutePath());
+    // Read home again using XML entry and save it in an other file
+    home = homeXmlEntryRecorder.readHome(savedFileWithXmlEntry.getAbsolutePath());
+    File savedFileWithXmlEntry2 = File.createTempFile("homeXML", ".sh3d");
+    homeXmlEntryRecorder.writeHome(home, savedFileWithXmlEntry2.getAbsolutePath());
+    // Compare the XML entries of the two files
+    assertContentEquals("Home.xml entries different", 
+        new URLContent(new URL("jar:" + savedFileWithXmlEntry.toURI().toURL() + "!/Home.xml")), 
+        new URLContent(new URL("jar:" + savedFileWithXmlEntry2.toURI().toURL() + "!/Home.xml")));
+    savedFileWithXmlEntry.delete();
+    savedFileWithXmlEntry2.delete();
+  }
+  
   /**
    * Test repaired home file management.
    */
@@ -94,7 +157,7 @@ public class HomeFileRecorderTest extends TestCase {
         HomeControllerTest.class.getResource("resources/damagedHomeInValidZipWithContentDigestsAndNoContent.sh3d").toURI()).getAbsolutePath(), 9);
   }
 
-  public void checkDamagedFileIsRepaired(String testFile, int damagedContentCount) throws RecorderException, IOException {
+  private void checkDamagedFileIsRepaired(String testFile, int damagedContentCount) throws RecorderException, IOException {
     try {
       // Check if opened home isn't repaired if preferences content isn't provided
       HomeRecorder recorder = new HomeFileRecorder(0, false, null, false);
@@ -109,7 +172,7 @@ public class HomeFileRecorderTest extends TestCase {
       Home home = recorder.readHome(testFile);
       assertTrue("Home is not flagged as repaired", home.isRepaired());
       // Check repaired home can be saved
-      File savedFile = File.createTempFile("repaired", "sh3d");
+      File savedFile = File.createTempFile("repaired", ".sh3d");
       recorder.writeHome(home, savedFile.getAbsolutePath());
       savedFile.delete();
     } catch (DamagedHomeRecorderException ex) {
@@ -140,6 +203,7 @@ public class HomeFileRecorderTest extends TestCase {
       assertFalse("Different wall at end", wall2.getWallAtEnd() == null);
       assertNotSame("Wall at end not loaded", wall1.getWallAtStart(), wall2.getWallAtEnd());
     }
+    assertEquals("Different property id", wall1.getProperty("id"), wall2.getProperty("id"));     
   }
 
   /**
@@ -159,10 +223,12 @@ public class HomeFileRecorderTest extends TestCase {
     assertContentEquals("Different icon content", piece1.getIcon(), piece2.getIcon());     
     assertNotSame("Piece model not loaded", piece1.getModel(), piece2.getModel());
     assertContentEquals("Different model content", piece1.getModel(), piece2.getModel());     
+    assertEquals("Different property id", piece1.getProperty("id"), piece2.getProperty("id"));     
+    assertEquals("Different property name", piece1.getProperty("name"), piece2.getProperty("name"));     
   }
 
   /**
-   * Asserts <code>content1</code> and <code>content2</code> are equals.
+   * Asserts <code>content1</code> and <code>content2</code> are equal.
    */
   private void assertContentEquals(String message, Content content1, Content content2) {
     InputStream stream1 = null;
